/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.control');

org.ametys.cms.control.TriggerField = function(id, configuration, pluginName)
{
	org.ametys.cms.control.TriggerField.superclass.constructor.call(this, id, configuration, pluginName);

	eval("this._selectionListener = " + this.getConfiguration()['selection-listener'] + ";");
}

Ext.extend(org.ametys.cms.control.TriggerField, org.ametys.ribbon.control.RibbonControl, {});

org.ametys.cms.control.TriggerField.prototype.createControl = function(size, colspan)
{
	var initFn = this.getConfiguration()['initialize'];
	if (initFn != null)
	{
		eval(initFn + "(this.getId(), this.getConfiguration(), this.getPluginName());");
	}
	
	var listeners = {};
	
	var conf = this.getConfiguration(); 
	
	listeners['render'] = function() {
		if (conf['default-description'] != '')
		{
			var tooltip = this.tooltip || org.ametys.fluent.Tooltip.create (
												conf['label'],
												conf['icon-large'] || conf['icon-medium'] || conf['icon-small'] || conf['icon'],
												conf['default-description'], 
												conf['footer']
											);

			if (Ext.isObject(tooltip))
			{
				Ext.QuickTips.register(Ext.apply({
						target: this.el.dom.id
					}, tooltip));
			
				if (this.label)
				{
					Ext.QuickTips.register(Ext.apply({
						target: this.label.id
					}, tooltip));
				}
			}
			else
			{
				this.el.dom['qtip'] = tooltip;
				if (this.label)
				{
					this.label.dom['qtip'] = tooltip;
				}
			}
		}
				
		if (conf['menu'])
		{
			this.menu = conf['menu'];
			if (Ext.isArray(this.menu)){
                this.menu = { items: this.menu };
            }
            
            if (Ext.isObject(this.menu)){
                this.menu.ownerCt = this;
            }
            
            this.menu = Ext.menu.MenuMgr.get(this.menu);
            this.menu.ownerCt = undefined;
		}
	}
	
	for (var key in this.getConfiguration())
	{
		if (/^on(.*)$/.test(key))
		{
			listeners[RegExp.$1] = eval(this.getConfiguration()[key]);
		}
	}
	
	var width = this._getWidth(size);
	var elementConf = this._getConf(width, listeners);
	
	var panel = new Ext.Panel ({
	       layout: "form",
	       defaultType : 'trigger',
	       labelAlign: size == 'large' && elementConf.fieldLabel ? 'top' : 'left',
	       labelWidth : this.getConfiguration()['label-width'] != null ? Number(this.getConfiguration()['label-width']) : 60,
	       colspan: colspan,
	    	
	       items:[elementConf]
	});

	var element = panel.items.get(0);
	if (this.getConfiguration().menu)
	{
		element.onTriggerClick = this._onTriggerClick.createDelegate(element);
	}
	
	this.getElements().add(element);
	
	return panel;
}

org.ametys.cms.control.TriggerField.prototype._onTriggerClick = function (event)
{
	if (this.rendered && this.menu)
	{
	    if (this.menu.isVisible())
	    {
	        this.menu.hide();
	    }
	    this.menu.ownerCt = this;
	    this.menu.show(arguments[0].getTarget(), 'tl-bl');
	}
}

org.ametys.cms.control.TriggerField.prototype._getWidth = function(size)
{
	var width = this.getConfiguration()['width'] != null ? Number(this.getConfiguration()['width']) : 'auto';
	width = this.getConfiguration()['width-' + size] != null ? Number(this.getConfiguration()['width-' + size]) : width;
	return width;
}
org.ametys.cms.control.TriggerField.prototype._getConf = function(width, listeners)
{
	var conf = {
		cls: this.getConfiguration()['cls'],
		readOnly: this.getConfiguration()['readOnly'] == "true",
			
		inputType: this.getConfiguration()['input-type'] != null ? this.getConfiguration()['input-type'] : 'text', 
    	xtype: this.getConfiguration()['input-xtype'] != null ? this.getConfiguration()['input-xtype'] : 'trigger', 

    	hideLabel: this.getConfiguration()['label'] == null,
    	allowNegative: this.getConfiguration()['allowNegative'] != null ? this.getConfiguration()['allowNegative'] == "true" : false,
    	allowDecimals: this.getConfiguration()['allowDecimals'] != null ? this.getConfiguration()['allowDecimals'] == "true" : false,
    	fieldLabel: this.getConfiguration()['label'],
    	name: this.getConfiguration()['name'],
    	height: this.getConfiguration()['height'],
    	emptyText : this.getConfiguration()['empty-text'],
    	triggerClass: this.getConfiguration()['triggerClass'] != null ? this.getConfiguration()['triggerClass'] : '',
    			
    	enableKeyEvents : true,
    	
    	width: width,
    	disabled: this.getConfiguration()['disabled'] == "true",
    	
    	listeners: listeners
    };
	
	if (this.getConfiguration()['triggerConfig'])
	{
		conf.triggerConfig = this.getConfiguration()['triggerConfig']
	}
	
	return conf;
}

/**
 * Get the messages to determine enable / disable
 * @param {org.ametys.messagebus.message.Message[]} messages The messages received
 */
org.ametys.cms.control.TriggerField.prototype.onMessage = function(messages)
{
	for (var i = 0; i < messages.length; i++)
	{
		var message = messages[i];
		if (message.getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = message.getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, function(target) { return target.getType() == "node";});

			var node = target != null ? target.getParameters()['object'] : null;

			if (this._selectionListener)
			{
				this._selectionListener(this, target, node);
			}
			else
			{
				this.getElements().each(function(element) { element.setDisabled(target == null); });
			}
		}
	}
}

